# workflow_engine/nodes/iteration.py
"""
Nodes that iterate over a sequence of items.
"""

from functools import cached_property
from typing import ClassVar, Literal, Self

from overrides import override
from workflow_engine import (
    Context,
    DataValue,
    Edge,
    InputNode,
    Node,
    NodeTypeInfo,
    OutputNode,
    Params,
    StringMapValue,
    ValueSchemaValue,
    Workflow,
    WorkflowValue,
)
from workflow_engine.core.io import SchemaParams
from workflow_engine.core.values.schema import SequenceValueSchema
from workflow_engine.nodes.data import (
    ExpandDataNode,
    ExpandSequenceNode,
    GatherDataNode,
    GatherSequenceNode,
    SequenceData,
)

from ..node_base import AceTeamNode


class ForEachParams(Params):
    workflow: WorkflowValue


class ForEachNode(AceTeamNode[SequenceData, SequenceData, ForEachParams]):
    """
    A node that executes the internal workflow W for each item in the input
    sequence.

    For each item i in the input sequence, create a copy of W, call it W[i].
    We expand the sequence into its individual data objects and expand
    sequence[i] into the input fields of W[i].
    Then, we gather the output of each W[i] into a single object, before
    gathering them further into a single sequence.

    The output of this node is a sequence of the same length as the input
    sequence, with each item being the output of the internal workflow.
    """

    TYPE_INFO: ClassVar[NodeTypeInfo] = NodeTypeInfo.from_parameter_type(
        name="ForEach",
        display_name="ForEach",
        description="Executes the internal workflow for each item in the input sequence.",
        version="0.4.0",
        parameter_type=ForEachParams,
    )

    type: Literal["ForEach"] = "ForEach"  # pyright: ignore[reportIncompatibleVariableOverride]

    @cached_property
    def workflow(self) -> Workflow:
        return self.params.workflow.root

    @cached_property
    def input_type(self):
        return SequenceData[DataValue[self.workflow.input_type]]

    @cached_property
    def output_type(self):
        return SequenceData[DataValue[self.workflow.output_type]]

    @override
    async def run(self, context: Context, input: SequenceData) -> Workflow:
        N = len(input.sequence)

        input_params = SchemaParams(
            fields=StringMapValue[ValueSchemaValue](
                {
                    "sequence": ValueSchemaValue(
                        SequenceValueSchema(
                            type="array",
                            items=self.workflow.input_schema,
                        )
                    ),
                }
            )
        )
        output_params = SchemaParams(
            fields=StringMapValue[ValueSchemaValue](
                {
                    "sequence": ValueSchemaValue(
                        SequenceValueSchema(
                            type="array",
                            items=self.workflow.output_schema,
                        )
                    ),
                }
            )
        )

        input_node = InputNode(id="input", params=input_params)
        output_node = OutputNode(id="output", params=output_params)

        inner_nodes: list[Node] = []
        edges: list[Edge] = []

        expand = ExpandSequenceNode.from_length(
            id="expand",
            length=N,
            element_type=DataValue[self.workflow.input_type],
        )
        gather = GatherSequenceNode.from_length(
            id="gather",
            length=N,
            element_type=DataValue[self.workflow.output_type],
        )
        inner_nodes.append(expand)
        inner_nodes.append(gather)

        # Connect input_node -> expand and gather -> output_node
        edges.append(
            Edge.from_nodes(
                source=input_node,
                source_key="sequence",
                target=expand,
                target_key="sequence",
            )
        )
        edges.append(
            Edge.from_nodes(
                source=gather,
                source_key="sequence",
                target=output_node,
                target_key="sequence",
            )
        )

        for i in range(N):
            namespace = f"element_{i}"
            input_adapter = ExpandDataNode.from_data_type(
                id="input_adapter",
                data_type=self.workflow.input_type,
            ).with_namespace(namespace)
            item_workflow = self.workflow.with_namespace(namespace)
            output_adapter = GatherDataNode.from_data_type(
                id="output_adapter",
                data_type=self.workflow.output_type,
            ).with_namespace(namespace)

            inner_nodes.append(input_adapter)
            inner_nodes.extend(item_workflow.inner_nodes)
            inner_nodes.append(output_adapter)

            edges.append(
                Edge.from_nodes(
                    source=expand,
                    source_key=expand.key(i),
                    target=input_adapter,
                    target_key="data",
                )
            )
            for edge in item_workflow.edges:
                if edge.source_id == item_workflow.input_node.id:
                    source_id = input_adapter.id
                else:
                    source_id = edge.source_id
                if edge.target_id == item_workflow.output_node.id:
                    target_id = output_adapter.id
                else:
                    target_id = edge.target_id
                edges.append(
                    Edge(
                        source_id=source_id,
                        source_key=edge.source_key,
                        target_id=target_id,
                        target_key=edge.target_key,
                    )
                )
            edges.append(
                Edge.from_nodes(
                    source=output_adapter,
                    source_key="data",
                    target=gather,
                    target_key=gather.key(i),
                )
            )

        return Workflow(
            input_node=input_node,
            inner_nodes=inner_nodes,
            output_node=output_node,
            edges=edges,
        )

    @classmethod
    def from_workflow(
        cls,
        id: str,
        workflow: Workflow,
    ) -> Self:
        return cls(id=id, params=ForEachParams(workflow=WorkflowValue(workflow)))


__all__ = [
    "ForEachNode",
]



"""Tests for individual nodes."""

import pytest
from workflow_engine import BooleanValue, Empty, FloatValue, StringValue
from workflow_engine.contexts.in_memory import InMemoryContext

from aceteam_nodes.nodes.comparison import (
    AndNode,
    ComparisonInput,
    ComparisonParams,
    EqualNode,
    GreaterThanNode,
    LessThanNode,
    LogicalInput,
    LogicalParams,
    NotInput,
    NotNode,
    OrNode,
)
from aceteam_nodes.nodes.csv_reader import CSVReaderNode, CSVReaderNodeParams
from aceteam_nodes.nodes.data_transform import (
    DataTransformNode,
    DataTransformNodeInput,
    DataTransformNodeParams,
)
from aceteam_nodes.nodes.text_io import TextInputNode, TextInputNodeParams


@pytest.fixture
def context():
    return InMemoryContext()


async def test_text_input_node(context):
    node = TextInputNode(
        id="test",
        params=TextInputNodeParams(text=StringValue("Hello World")),
    )
    output = await node.run(context, Empty())
    assert output.output.root == "Hello World"


async def test_data_transform_passthrough(context):
    node = DataTransformNode(
        id="test",
        params=DataTransformNodeParams(instruction=StringValue("Pass through")),
    )
    input_data = DataTransformNodeInput(input=StringValue("test data"))
    output = await node.run(context, input_data)
    assert output.output.root == "test data"


async def test_csv_reader_node(context):
    node = CSVReaderNode(
        id="test",
        params=CSVReaderNodeParams(sample_data=StringValue("name,age\nAlice,30\nBob,25")),
    )
    output = await node.run(context, Empty())
    assert "Alice" in output.data.root
    assert "Bob" in output.data.root


async def test_equal_node(context):
    node = EqualNode(id="test", params=ComparisonParams())
    result = await node.run(context, ComparisonInput(a=FloatValue(5.0), b=FloatValue(5.0)))
    assert result.result.root is True

    result = await node.run(context, ComparisonInput(a=FloatValue(5.0), b=FloatValue(3.0)))
    assert result.result.root is False


async def test_greater_than_node(context):
    node = GreaterThanNode(id="test", params=ComparisonParams())
    result = await node.run(context, ComparisonInput(a=FloatValue(5.0), b=FloatValue(3.0)))
    assert result.result.root is True

    result = await node.run(context, ComparisonInput(a=FloatValue(3.0), b=FloatValue(5.0)))
    assert result.result.root is False


async def test_less_than_node(context):
    node = LessThanNode(id="test", params=ComparisonParams())
    result = await node.run(context, ComparisonInput(a=FloatValue(3.0), b=FloatValue(5.0)))
    assert result.result.root is True


async def test_and_node(context):
    node = AndNode(id="test", params=LogicalParams())
    result = await node.run(context, LogicalInput(a=BooleanValue(True), b=BooleanValue(True)))
    assert result.result.root is True

    result = await node.run(context, LogicalInput(a=BooleanValue(True), b=BooleanValue(False)))
    assert result.result.root is False


async def test_or_node(context):
    node = OrNode(id="test", params=LogicalParams())
    result = await node.run(context, LogicalInput(a=BooleanValue(False), b=BooleanValue(True)))
    assert result.result.root is True

    result = await node.run(context, LogicalInput(a=BooleanValue(False), b=BooleanValue(False)))
    assert result.result.root is False


async def test_not_node(context):
    node = NotNode(id="test", params=LogicalParams())
    result = await node.run(context, NotInput(a=BooleanValue(True)))
    assert result.result.root is False

    result = await node.run(context, NotInput(a=BooleanValue(False)))
    assert result.result.root is True



"""AceTeam node base class with auto-registration."""

from typing import Generic, TypeVar

from workflow_engine import Data, Node, Params
from workflow_engine.core import NodeTypeInfo as WENodeTypeInfo

Input_contra = TypeVar("Input_contra", bound=Data, contravariant=True)
Output_co = TypeVar("Output_co", bound=Data, covariant=True)
Params_co = TypeVar("Params_co", bound=Params, covariant=True)


class AceTeamNode(
    Node[Input_contra, Output_co, Params_co],
    Generic[Input_contra, Output_co, Params_co],
):
    """
    An AceTeam-specific node with field information for dynamic input/output types.
    """

    TYPE_INFO = WENodeTypeInfo.from_parameter_type(
        name="AceTeamNode",
        display_name="AceTeam Node",
        version="0.4.0",
        parameter_type=Params,
    )

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        if cls.__name__.split("[")[0] != "AceTeamNode":
            assert cls not in aceteam_nodes
            aceteam_nodes.append(cls)


aceteam_nodes: list[type[AceTeamNode]] = []


__all__ = [
    "AceTeamNode",
    "aceteam_nodes",
]



"""Comparison and logical operator nodes."""

from functools import cached_property
from typing import Literal

from overrides import override
from workflow_engine import BooleanValue, Context, Data, FloatValue, NodeTypeInfo, Params

from ..node_base import AceTeamNode


class ComparisonParams(Params):
    pass


class ComparisonInput(Data):
    a: FloatValue
    b: FloatValue


class ComparisonOutput(Data):
    result: BooleanValue


class LogicalParams(Params):
    pass


class LogicalInput(Data):
    a: BooleanValue
    b: BooleanValue


class NotInput(Data):
    a: BooleanValue


class LogicalOutput(Data):
    result: BooleanValue


# --- Comparison Nodes ---


class EqualNode(AceTeamNode[ComparisonInput, ComparisonOutput, ComparisonParams]):
    """Checks if a == b."""

    TYPE_INFO = NodeTypeInfo.from_parameter_type(
        name="Equal",
        display_name="Equal",
        description="Checks if two numbers are equal (a == b).",
        version="1.0.0",
        parameter_type=ComparisonParams,
    )
    type: Literal["Equal"] = "Equal"

    @cached_property
    def input_type(self):
        return ComparisonInput

    @cached_property
    def output_type(self):
        return ComparisonOutput

    @override
    async def run(self, context: Context, input: ComparisonInput) -> ComparisonOutput:
        return ComparisonOutput(result=BooleanValue(input.a.root == input.b.root))


class NotEqualNode(AceTeamNode[ComparisonInput, ComparisonOutput, ComparisonParams]):
    """Checks if a != b."""

    TYPE_INFO = NodeTypeInfo.from_parameter_type(
        name="NotEqual",
        display_name="Not Equal",
        description="Checks if two numbers are not equal (a != b).",
        version="1.0.0",
        parameter_type=ComparisonParams,
    )
    type: Literal["NotEqual"] = "NotEqual"

    @cached_property
    def input_type(self):
        return ComparisonInput

    @cached_property
    def output_type(self):
        return ComparisonOutput

    @override
    async def run(self, context: Context, input: ComparisonInput) -> ComparisonOutput:
        return ComparisonOutput(result=BooleanValue(input.a.root != input.b.root))


class GreaterThanNode(AceTeamNode[ComparisonInput, ComparisonOutput, ComparisonParams]):
    """Checks if a > b."""

    TYPE_INFO = NodeTypeInfo.from_parameter_type(
        name="GreaterThan",
        display_name="Greater Than",
        description="Checks if a > b.",
        version="1.0.0",
        parameter_type=ComparisonParams,
    )
    type: Literal["GreaterThan"] = "GreaterThan"

    @cached_property
    def input_type(self):
        return ComparisonInput

    @cached_property
    def output_type(self):
        return ComparisonOutput

    @override
    async def run(self, context: Context, input: ComparisonInput) -> ComparisonOutput:
        return ComparisonOutput(result=BooleanValue(input.a.root > input.b.root))


class GreaterThanEqualNode(AceTeamNode[ComparisonInput, ComparisonOutput, ComparisonParams]):
    """Checks if a >= b."""

    TYPE_INFO = NodeTypeInfo.from_parameter_type(
        name="GreaterThanEqual",
        display_name="Greater Than or Equal",
        description="Checks if a >= b.",
        version="1.0.0",
        parameter_type=ComparisonParams,
    )
    type: Literal["GreaterThanEqual"] = "GreaterThanEqual"

    @cached_property
    def input_type(self):
        return ComparisonInput

    @cached_property
    def output_type(self):
        return ComparisonOutput

    @override
    async def run(self, context: Context, input: ComparisonInput) -> ComparisonOutput:
        return ComparisonOutput(result=BooleanValue(input.a.root >= input.b.root))


class LessThanNode(AceTeamNode[ComparisonInput, ComparisonOutput, ComparisonParams]):
    """Checks if a < b."""

    TYPE_INFO = NodeTypeInfo.from_parameter_type(
        name="LessThan",
        display_name="Less Than",
        description="Checks if a < b.",
        version="1.0.0",
        parameter_type=ComparisonParams,
    )
    type: Literal["LessThan"] = "LessThan"

    @cached_property
    def input_type(self):
        return ComparisonInput

    @cached_property
    def output_type(self):
        return ComparisonOutput

    @override
    async def run(self, context: Context, input: ComparisonInput) -> ComparisonOutput:
        return ComparisonOutput(result=BooleanValue(input.a.root < input.b.root))


class LessThanEqualNode(AceTeamNode[ComparisonInput, ComparisonOutput, ComparisonParams]):
    """Checks if a <= b."""

    TYPE_INFO = NodeTypeInfo.from_parameter_type(
        name="LessThanEqual",
        display_name="Less Than or Equal",
        description="Checks if a <= b.",
        version="1.0.0",
        parameter_type=ComparisonParams,
    )
    type: Literal["LessThanEqual"] = "LessThanEqual"

    @cached_property
    def input_type(self):
        return ComparisonInput

    @cached_property
    def output_type(self):
        return ComparisonOutput

    @override
    async def run(self, context: Context, input: ComparisonInput) -> ComparisonOutput:
        return ComparisonOutput(result=BooleanValue(input.a.root <= input.b.root))


# --- Logical Nodes ---


class AndNode(AceTeamNode[LogicalInput, LogicalOutput, LogicalParams]):
    """Logical AND (a && b)."""

    TYPE_INFO = NodeTypeInfo.from_parameter_type(
        name="And",
        display_name="Logical AND",
        description="Logical AND (a && b).",
        version="1.0.0",
        parameter_type=LogicalParams,
    )
    type: Literal["And"] = "And"

    @cached_property
    def input_type(self):
        return LogicalInput

    @cached_property
    def output_type(self):
        return LogicalOutput

    @override
    async def run(self, context: Context, input: LogicalInput) -> LogicalOutput:
        return LogicalOutput(result=BooleanValue(input.a.root and input.b.root))


class OrNode(AceTeamNode[LogicalInput, LogicalOutput, LogicalParams]):
    """Logical OR (a || b)."""

    TYPE_INFO = NodeTypeInfo.from_parameter_type(
        name="Or",
        display_name="Logical OR",
        description="Logical OR (a || b).",
        version="1.0.0",
        parameter_type=LogicalParams,
    )
    type: Literal["Or"] = "Or"

    @cached_property
    def input_fields_info(self):
        return LogicalInput

    @property
    def output_type(self):
        return LogicalOutput

    @override
    async def run(self, context: Context, input: LogicalInput) -> LogicalOutput:
        return LogicalOutput(result=BooleanValue(input.a.root or input.b.root))


class NotNode(AceTeamNode[NotInput, LogicalOutput, LogicalParams]):
    """Logical NOT (!a)."""

    TYPE_INFO = NodeTypeInfo.from_parameter_type(
        name="Not",
        display_name="Logical NOT",
        description="Logical NOT (!a).",
        version="1.0.0",
        parameter_type=LogicalParams,
    )
    type: Literal["Not"] = "Not"

    @cached_property
    def input_type(self):
        return NotInput

    @cached_property
    def output_type(self):
        return LogicalOutput

    @override
    async def run(self, context: Context, input: NotInput) -> LogicalOutput:
        return LogicalOutput(result=BooleanValue(not input.a.root))


__all__ = [
    "AndNode",
    "EqualNode",
    "GreaterThanEqualNode",
    "GreaterThanNode",
    "LessThanEqualNode",
    "LessThanNode",
    "NotEqualNode",
    "NotNode",
    "OrNode",
]



"""Text Input node - simple text source for workflows."""

from functools import cached_property
from typing import Literal

from overrides import override
from pydantic import Field
from workflow_engine import Context, Data, Empty, Params, StringValue
from workflow_engine.core import NodeTypeInfo as WENodeTypeInfo

from ..node_base import AceTeamNode


class TextInputNodeParams(Params):
    text: StringValue = Field(
        default=StringValue(""),
        description="The text content to output",
    )


class TextInputNodeOutput(Data):
    output: StringValue


class TextInputNode(
    AceTeamNode[Empty, TextInputNodeOutput, TextInputNodeParams],
):
    """A simple text source node that outputs a configurable text string."""

    TYPE_INFO = WENodeTypeInfo.from_parameter_type(
        name="TextInput",
        display_name="Text Input",
        description="A simple text source.",
        version="1.0.0",
        parameter_type=TextInputNodeParams,
    )

    type: Literal["TextInput"] = "TextInput"

    @cached_property
    def output_type(self):
        return TextInputNodeOutput

    @override
    async def run(self, context: Context, input: Empty) -> TextInputNodeOutput:
        return TextInputNodeOutput(output=self.params.text)


__all__ = [
    "TextInputNode",
    "TextInputNodeOutput",
    "TextInputNodeParams",
]



"""Data Transform node - simple data transformation."""

from functools import cached_property
from typing import Literal

from overrides import override
from pydantic import Field
from workflow_engine import Context, Data, Params, StringValue
from workflow_engine.core import NodeTypeInfo as WENodeTypeInfo

from ..node_base import AceTeamNode


class DataTransformNodeParams(Params):
    instruction: StringValue = Field(
        default=StringValue("Pass through unchanged"),
        description="Description of the transformation to apply",
    )


class DataTransformNodeInput(Data):
    input: StringValue = Field(description="The data to transform")


class DataTransformNodeOutput(Data):
    output: StringValue = Field(description="The transformed data")


class DataTransformNode(
    AceTeamNode[DataTransformNodeInput, DataTransformNodeOutput, DataTransformNodeParams],
):
    """Processes and transforms input data based on configured instructions."""

    TYPE_INFO = WENodeTypeInfo.from_parameter_type(
        name="DataTransform",
        display_name="Transform",
        description="Processes and transforms data.",
        version="1.0.0",
        parameter_type=DataTransformNodeParams,
    )

    type: Literal["DataTransform"] = "DataTransform"

    @cached_property
    def input_type(self):
        return DataTransformNodeInput

    @cached_property
    def output_type(self):
        return DataTransformNodeOutput

    @override
    async def run(self, context: Context, input: DataTransformNodeInput) -> DataTransformNodeOutput:
        # Pass through - transformation logic can be extended
        return DataTransformNodeOutput(output=input.input)


__all__ = [
    "DataTransformNode",
    "DataTransformNodeInput",
    "DataTransformNodeOutput",
    "DataTransformNodeParams",
]



"""AceTeam workflow nodes.

Node submodules are NOT auto-imported here to avoid registration conflicts
when this package is used as a library alongside a platform that defines
its own versions of some nodes (e.g., APICallNode, LLMNode).

Import specific node modules directly:
    from aceteam_nodes.nodes.comparison import EqualNode
    from aceteam_nodes.nodes.conditional import IfNode

Or call register_all_nodes() to trigger auto-registration of all nodes
(e.g., before deserializing a workflow file).
"""

from workflow_engine import Node, NodeRegistry

from ..node_base import AceTeamNode
from .api_call import APICallNode
from .comparison import (
    AndNode,
    EqualNode,
    GreaterThanEqualNode,
    GreaterThanNode,
    LessThanEqualNode,
    LessThanNode,
    NotEqualNode,
    NotNode,
    OrNode,
)
from .conditional import IfElseNode, IfNode
from .csv_reader import CSVReaderNode
from .data_transform import DataTransformNode
from .iteration import ForEachNode
from .llm import LLMNode
from .text_io import TextInputNode

aceteam_node_registry = (
    NodeRegistry.builder()
    .register_base_node_class(Node)
    .register_base_node_class(AceTeamNode)
    .register_node_class("APICall", APICallNode)
    .register_node_class("And", AndNode)
    .register_node_class("CSVReader", CSVReaderNode)
    .register_node_class("DataTransform", DataTransformNode)
    .register_node_class("Equal", EqualNode)
    .register_node_class("GreaterThan", GreaterThanNode)
    .register_node_class("GreaterThanEqual", GreaterThanEqualNode)
    .register_node_class("If", IfNode)
    .register_node_class("IfElse", IfElseNode)
    .register_node_class("Iteration", ForEachNode)
    .register_node_class("LessThan", LessThanNode)
    .register_node_class("LessThanEqual", LessThanEqualNode)
    .register_node_class("LLM", LLMNode)
    .register_node_class("Not", NotNode)
    .register_node_class("NotEqual", NotEqualNode)
    .register_node_class("Or", OrNode)
    .register_node_class("TextInput", TextInputNode)
).build()



"""Field types and field info models for AceTeam nodes."""

from __future__ import annotations

from collections.abc import Mapping, Sequence
from csv import DictReader, DictWriter
from io import StringIO
from typing import Any, ClassVar, TypeVar

from workflow_engine import Context, Data
from workflow_engine.core.values.value import ValueRegistry
from workflow_engine.files import TextFileValue

# Value subclasses auto-register in workflow_engine's value type registry.
# When used alongside a platform that defines the same types (e.g., aceteam-8),
# we reuse the already-registered class to avoid duplicate registration errors.


def _get_or_define_value_type(name: str, cls_factory):
    """Return existing registered class or define a new one."""
    cls = ValueRegistry.DEFAULT.get_value_class(name)
    if cls is not None:
        return cls
    return cls_factory()


def _make_csv_file_value():
    class CSVFileValue(TextFileValue):
        """A CSV file."""

        mime_type: ClassVar[str] = "text/csv"

        async def read_data(self, context: Context) -> Sequence[Mapping[str, Any]]:
            text = await self.read_text(context)
            text_io = StringIO(text)
            reader = DictReader(text_io)
            return tuple(reader)

        async def write_data(self, context: Context, data: Sequence[Mapping[str, Any]]):
            text_io = StringIO()
            writer = DictWriter(text_io, fieldnames=data[0].keys())
            writer.writeheader()
            writer.writerows(data)
            text = text_io.getvalue()
            return await self.write_text(context, text)

    return CSVFileValue


CSVFileValue = _get_or_define_value_type("CSVFileValue", _make_csv_file_value)


D = TypeVar("D", bound=Data)


__all__ = [
    "CSVFileValue",
]



"""CSV Reader node - reads CSV data."""

from functools import cached_property
from typing import Literal

from overrides import override
from pydantic import Field
from workflow_engine import Context, Data, Empty, Params, StringValue
from workflow_engine.core import NodeTypeInfo as WENodeTypeInfo

from ..node_base import AceTeamNode


class CSVReaderNodeParams(Params):
    sample_data: StringValue = Field(
        default=StringValue("name,value\nexample,1"),
        description="Embedded CSV data",
    )


class CSVReaderNodeOutput(Data):
    data: StringValue = Field(
        description="The CSV data as text",
    )


class CSVReaderNode(
    AceTeamNode[Empty, CSVReaderNodeOutput, CSVReaderNodeParams],
):
    """Reads CSV data and outputs it as text."""

    TYPE_INFO = WENodeTypeInfo.from_parameter_type(
        name="CSVReader",
        display_name="CSV Reader",
        description="Reads CSV data and outputs it as text.",
        version="1.0.0",
        parameter_type=CSVReaderNodeParams,
    )

    type: Literal["CSVReader"] = "CSVReader"

    @cached_property
    def output_type(self):
        return CSVReaderNodeOutput

    @override
    async def run(self, context: Context, input: Empty) -> CSVReaderNodeOutput:
        return CSVReaderNodeOutput(data=self.params.sample_data)


__all__ = [
    "CSVReaderNode",
    "CSVReaderNodeOutput",
    "CSVReaderNodeParams",
]
